(* Wetter-Daten bearbeiten

    2007-2023 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: kontakt(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   *)

unit EditVal;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, NumberEd;

const
  NoVal = -1E6;
    
type
  TVarProp = record
    Name,VUnit : string;
    Digits,Decimal : integer;
    Min,Max,Delta : double;
    end;

  TEditValDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    btnCopyLast: TBitBtn;
    lblName: TLabel;
    btnCopyNext: TBitBtn;
    btnCopyBoth: TBitBtn;
    edtLast: TFloatEdit;
    edtNext: TFloatEdit;
    edtVal: TFloatComboBox;
    procedure btnCopyLastClick(Sender: TObject);
    procedure btnCopyNextClick(Sender: TObject);
    procedure btnCopyBothClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function Execute (Pos       : TPoint;
                      VarProp   : TVarProp;
                      Last,Next : double;
                      var AVal  : double) : boolean;
  end;

var
  EditValDialog: TEditValDialog;

implementation

{$R *.DFM}

function TEditValDialog.Execute (Pos       : TPoint;
                                 VarProp   : TVarProp;
                                 Last,Next : double;
                                 var AVal  : double) : boolean;
begin
  with Pos do begin
    Left:=X+20; Top:=Y;
    end;
  with VarProp do begin
    lblName.Caption:=Name+' in '+VUnit;
    end;
  if Last<=NoVal then begin
    edtLast.Text:=''; btnCopyLast.Enabled:=false;
    end
  else with edtLast do begin
    Value:=Last; Digits:=VarProp.Digits; Decimal:=VarProp.Decimal;
    btnCopyLast.Enabled:=true;
    end;
  if Next<=NoVal then begin
    edtNext.Text:=''; btnCopyNext.Enabled:=false;
    end
  else with edtNext do begin
    Value:=Next; Digits:=VarProp.Digits; Decimal:=VarProp.Decimal;
    btnCopyNext.Enabled:=true;
    end;
  btnCopyBoth.Enabled:=btnCopyLast.Enabled and btnCopyNext.Enabled;
  with edtVal do begin
    Value:=AVal; Digits:=VarProp.Digits; Decimal:=VarProp.Decimal;
    end;
  if ShowModal=mrOK then with edtVal do begin
    AVal:=Value; AddValue(Value);
    Result:=true;
    end
  else Result:=false;
  end;

procedure TEditValDialog.btnCopyLastClick(Sender: TObject);
begin
  edtVal.Value:=edtLast.Value;
  end;

procedure TEditValDialog.btnCopyNextClick(Sender: TObject);
begin
  edtVal.Value:=edtNext.Value;
  end;

procedure TEditValDialog.btnCopyBothClick(Sender: TObject);
begin
  edtVal.Value:=(edtLast.Value+edtNext.Value)/2;
  end;

end.
