(* Grafische Darstellung von Wetter-Tagesdaten

    2007-2024 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: kontakt(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 2  : Nov. 2013 (transferred to Delphi XE2)
   Vers. 3.0: Jan. 2017 (transferred to Delphi 10)
         3.2: Jan. 2018 - MinMax-Fenster
         3.3: Jun. 2018 - Anzeige eines Vergleichsmonats
         3.4: Aug. 2021 - Anzeige der Jahreswerte
         3.5: Dez. 2022 - Achsenbeschriftung zwischen den Skalenmarken
         3.6: Jan. 2023 - Anzeige der Messwerte beim Darberfahren mit der Maus
                          X-Achse mit monatsspezifischer Beschriftung
         3.7: Feb. 2024 - Anzeige von Schneefall

   letzte nderung: Februar 2024
   *)

unit Wetmain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Menus, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.ExtCtrls, System.IniFiles, System.Math, Vcl.Printers, Vcl.ComCtrls,
  WinPlot, HListBox, WinApiUtils, WinUtils, MsgDialogs,
  MonWahl, WetSub, WetHinw, WetEdit, WetNeu, WetListe, System.Classes,
  System.ImageList, Vcl.ImgList;

const
  ProgName = 'WinWet';
  Vers = ' - Vers. 3.7';

  sn='Summe des Niederschlags = ';
  sm='Mittelwerte: ';
  TSize = 0.4;   (* Textgre berschrift, Megren *)
  USize = 0.25;   (* Textgre Einheiten*)
  MSize = 0.3;   (* Textgre Mittelwerte, Summe *)
  OriName : array [poPortrait..poLandscape] of string =
           (' - Hochformat',' - QuerFormat');

{$I JrGlobal.pas}

type
  TPagePos = (poOben,poUnten);
  TGrafMode = (grScreen,grPrinter);
  TDataSet = (dsMain,dsComp,dsPrint);
  TLineType = (ltNone,ltTMax,ltTemp,ltTMin,ltPrs,ltRain);

  TJahresDaten = record
    Loaded : boolean;
    DatJahr  : integer;
    DatName  : string;
    LastPreviousDay,FirstNextDay : TDatRecTyp;
    DatenBuffer     : TDatBufTyp;
    end;

  TData = array [TDataSet] of TJahresDaten;

  RangeRecTyp = record
    Min,Max : double;
    end;

const
  TempRange : RangeRecTyp = (Min : -15.0; Max : 35.0);
  DruckRange : RangeRecTyp = (Min : 960; Max : 1060);
  RegenRange : RangeRecTyp = (Min : 0.0; Max : 50.0);

  LineColors : array[TLineType] of TColor =
    (clWhite,TColor($0000FF),TColor($00B200),TColor($CC0000),TColor($C400C4),TColor($808000));

type
  THauptForm = class(TForm)
    HauptMenu: TMainMenu;
    DatenMenu: TMenuItem;
    itmNeu: TMenuItem;
    itmEdit: TMenuItem;
    itmSave: TMenuItem;
    AnsichtMenu: TMenuItem;
    DruckenMenu: TMenuItem;
    Hilfe1: TMenuItem;
    GrafikBox: TScrollBox;
    GrafikFeld: TPaintBox;
    aktMonat1: TMenuItem;
    ganzesJahr1: TMenuItem;
    Druckereinstellung1: TMenuItem;
    Anpassen1: TMenuItem;
    N111: TMenuItem;
    N121: TMenuItem;
    N141: TMenuItem;
    itmDir: TMenuItem;
    N2: TMenuItem;
    itmDaten: TMenuItem;
    MehrereMonate1: TMenuItem;
    PrinterSetupDialog: TPrinterSetupDialog;
    paTools: TPanel;
    paSelect: TPanel;
    MonatVor: TSpeedButton;
    QuartalVor: TSpeedButton;
    MonatZur: TSpeedButton;
    QuartalZur: TSpeedButton;
    JahrZur: TSpeedButton;
    JahrVor: TSpeedButton;
    Januar: TSpeedButton;
    Dezember: TSpeedButton;
    MonatWahl: TComboBox;
    JahrWahl: THistoryCombo;
    paEdit: TPanel;
    DatNeu: TSpeedButton;
    DatBearb: TSpeedButton;
    DatDruck: TSpeedButton;
    EndeBtn: TSpeedButton;
    DatView: TSpeedButton;
    N1: TMenuItem;
    Ende1: TMenuItem;
    paGlobal: TPanel;
    StatusBar: TStatusBar;
    N3: TMenuItem;
    itmMinMax: TMenuItem;
    VJahrZur: TSpeedButton;
    VJahrVor: TSpeedButton;
    VJahrWahl: THistoryCombo;
    cbCompare: TCheckBox;
    AktMonat: TSpeedButton;
    MinMaxView: TSpeedButton;
    Jahreswerte: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    laTemp: TLabel;
    laRain: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    laVJahr: TLabel;
    bhPlotSheet: TBalloonHint;
    ilSymbols: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure NeuGraf(Sender: TObject);
    procedure DrEinstell(Sender: TObject);
    procedure Drucken(Sender: TObject);
    procedure EndeClick(Sender: TObject);
    procedure MonateDrucken (var ADaten : TJahresDaten; AJahr,Monat1,Monat2 : integer);
    procedure JahrDrucken(Sender: TObject);
    procedure AnderJahrClick(Sender: TObject);
    procedure NeuerMonat(Sender: TObject);
    procedure Mon3Zur(Sender: TObject);
    procedure Mon3Vor(Sender: TObject);
    procedure MonIncr(Sender: TObject);
    procedure MonDecr(Sender: TObject);
    procedure ShowMonat(Sender: TObject);
    procedure JahrEingabe(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Anpassen1Click(Sender: TObject);
    procedure N111Click(Sender: TObject);
    procedure N121Click(Sender: TObject);
    procedure N141Click(Sender: TObject);
    procedure SetboxSizes(Sender: TObject);
    procedure JahrDecr(Sender: TObject);
    procedure JahrInc(Sender: TObject);
    procedure PfadSetzen(Sender: TObject);
    procedure DataEdit(Sender: TObject);
    procedure ResetChecked (Menu : TMenuItem);
    procedure InfoMessage(Sender: TObject);
    procedure NeuesJahrClick(Sender: TObject);
    procedure JanuarClick(Sender: TObject);
    procedure DezemberClick(Sender: TObject);
    procedure ListeAnzeigen(Sender: TObject);
    procedure TeilDrucken(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure itmMinMaxClick(Sender: TObject);
    procedure itmSaveClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MinMaxViewClick(Sender: TObject);
    procedure cbCompareClick(Sender: TObject);
    procedure VJahrZurClick(Sender: TObject);
    procedure VJahrVorClick(Sender: TObject);
    procedure VJahrWahlKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure AktMonatClick(Sender: TObject);
    procedure GrafikFeldMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private-Deklarationen }
    ProgVersName,ProgVersDate,
    PrgPath,PrgName,
    IniName,AppPath,
    DocPath,DatPath : string;
    VersInfo        : TFileVersionInfo;
    PrtName         : string;
    PrtNdx,
    FormLeft,
    FormTop,
    FormWidth,
    FormHeight      : integer;
    FWindowState    : TWindowState;
    LTag,VJahr,PJahr : word;
    PrSize          : TFPoint;
    GrRand          : TFRect;
    mmVis           : boolean;
    mmPos,
    GrSize,
    StSize          : TPoint;
//    VDaten,
//    Daten           : TJahresDaten;
    Daten           : TData;
    PrPlot,GrPlot   : TPlot;

    (* INI-File einlesen *)
    procedure ReadIni;

    (* INI-File schreiben *)
    procedure WriteIni;

    (* Daten laden *)
    function DatenLaden (Pfad : string;
                         Jahr : integer;
                         Neu  : boolean;
                         ADatSet : TDataSet) : boolean;

    (* Daten sichern *)
    procedure DatenSichern;

    procedure ShowAverage (ADatSet : TDataSet);
    procedure MinMax (AShow : boolean);
    procedure EnableCompare (AEnabled : boolean);

    (* Gre des Zeichenfeldes festlegen *)
    procedure InitPlotField (var Plot     : TPlot;
                             Rand         : TFRect;
                             PagePos      : TPagePos);

    (* Diagramm malen *)
    procedure Diagramm (Plot     : TPlot;
                        var WDaten : TJahresdaten;
                        AJahr,
                        Monat    : word;
                        Clear    : boolean = true;
                        WithAxis : boolean = true);

  public
    { Public-Deklarationen }
    Jahr,Monat      : word;
    procedure MonatAnzeigen (m,j : integer);
    procedure MinMaxClosed;
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.DFM}

uses WinShell, PathUtils, Winapi.ShlObj, ShellDirDlg, System.DateUtils, Vcl.FileCtrl,
  MinMaxDlg, NumberUtils, JahrWahl;

const
  IniExt = 'ini';
  DefPath = 'Wetter';

  PrtSekt = 'Drucker';
  DatSekt = 'Daten';
  BildSekt = 'Form';
  MinMaxSekt = 'MinMax';

{-----------------------------------------------------------------------}
(* INI-File einlesen *)
procedure THauptForm.ReadIni;
var
  IniFile : TIniFile;
begin
  PrgPath:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  DocPath:=SetDirName(GetDesktopFolder(CSIDL_PERSONAL))+DefPath;
  DatPath:=DocPath;
  AppPath:=GetDesktopFolder(CSIDL_APPDATA);
  if length(AppPath)>0 then begin
    AppPath:=SetDirName(AppPath)+AppSubDir;  // Pfad zu Anwendungsdaten
    if not System.SysUtils.ForceDirectories(AppPath) then AppPath:=DatPath;
    end
  else AppPath:=DatPath;
// Versions-Info
  with VersInfo do if GetFileVersion (Application.ExeName,VersInfo) then begin
    // Versionsnr. ohne "Compilierung"
    Version:=DelExt(Version);
    Comments:=' (Vers. '+Version+')';
    end
  else begin
    InternalName:=ProgName; Version:=Vers; Copyright:=CopRgt;
    end;
  with VersInfo do ProgVersName:=InternalName+Comments;
  ProgVersDate:=DateToStr(FileDateToDateTime(FileAge(Application.ExeName)));
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    PrtNdx:=ReadInteger (PrtSekt,'PrtIndex',-1);
    FormLeft:=ReadInteger (BildSekt,'Left',0);
    FormTop:=ReadInteger (BildSekt,'Top',0);
    FormWidth:=ReadInteger (BildSekt,'Width',500);
    FormHeight:=ReadInteger (BildSekt,'Height',400);
    FWindowState:=TWindowState(ReadInteger (BildSekt,'State',Ord(wsNormal)));
    Jahr:=ReadInteger (DatSekt,'Jahr',YearOf(Date));
    Monat:=ReadInteger (DatSekt,'Monat',MonthOf(Date));
    VJahr:=ReadInteger (DatSekt,'VergleichsJahr',Jahr-1);
    DatPath:=SetDirName(ReadString(DatSekt,'DatenPfad',DatPath));
    with mmPos do begin
      x:=ReadInteger (MinMaxSekt,'Left',0);
      y:=ReadInteger (MinMaxSekt,'Top',0);
      end;
    mmVis:=ReadBool(MinMaxSekt,'Visible',true);
    Free;
    end;
  LTag:=1; PJahr:=Jahr;
  end;

(* INI-File schreiben *)
procedure THauptForm.WriteIni;
var
  IniFile : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteInteger (PrtSekt,'PrtIndex',PrtNdx);
    WriteInteger (BildSekt,'Left',Left);
    WriteInteger (BildSekt,'Top',Top);
    WriteInteger (BildSekt,'Width',ClientWidth);
    WriteInteger (BildSekt,'Height',ClientHeight);
    WriteInteger (BildSekt,'State',ord(WindowState));
    WriteInteger (DatSekt,'Jahr',Jahr);
    WriteInteger (DatSekt,'Monat',Monat);
    WriteInteger (DatSekt,'VergleichsJahr',VJahr);
    WriteString(DatSekt,'DatenPfad',DatPath);
    with MinMaxDialog do begin
      WriteInteger (MinMaxSekt,'Left',Left);
      WriteInteger (MinMaxSekt,'Top',Top);
      end;
    WriteBool(MinMaxSekt,'Visible',mmVis);
    Free;
    end;
  end;

(* Hauptform erstellen *)
procedure THauptForm.FormCreate(Sender: TObject);
var
  i : integer;
begin
  ReadIni;
  Caption:='Wetterdaten anzeigen und bearbeiten (Vers. '+VersInfo.Version+')';
  with JahrWahl do begin
    IniFileName:=IniName; Loaded;
    Text:=IntToStr(Jahr); AddItem(Text);
    end;
  with VJahrWahl do begin
    IniFileName:=IniName; Loaded;
    for i:=Items.Count-1 downto 0 do
      if not FileExists(DatPath+WDatName+Items[i]+WDatExt) then
        DeleteItem(Items[i]);
    Text:=IntToStr(VJahr);
    end;
  EnableCompare(false);
  GrPlot:=TPlot.Create (GrafikFeld.Canvas);
  with Printer do begin
    (* PrinterIndex:=0;*)
    Orientation:=poPortrait;
    end;
  with GrPlot do begin
    with PrSize do GetPaperSize (Printer,X,Y);
    GetDefMargins (Printer.Orientation,GrRand);
    end;
  SetBoxSizes (Sender);
  GrSize:=StSize;
  Anpassen1.Checked:=true;
  InitPlotField (GrPlot,GrRand,poOben);
  Daten[dsMain].Loaded:=false;
  Daten[dsComp].Loaded:=false;
  end;

procedure THauptForm.FormShow(Sender: TObject);
begin
  ShellDirDialog.LoadFromIni(IniName,'Pfade');
  Left:=FormLeft; Top:=FormTop;
  ClientWidth:=FormWidth; ClientHeight:=FormHeight;
  WindowState:=FWindowState;
  if mmPos.X=0 then mmPos:=ClientToScreen(Point(EndeBtn.Left-200,GrafikBox.Top+10));
  with mmPos,MinMaxDialog do begin
    if x<0 then x:=10
    else if x+Width>Screen.Width then x:=Screen.Width-Width-10;
    if y<0 then y:=10
    else if y+Height>Screen.Height then y:=Screen.Height-Height-10;
    Left:=x; Top:=y;
    end;
  MinMaxView.Down:=mmVis;
  itmMinMax.Checked:=mmVis;
  MinMax(mmVis);
  DatenLaden (DatPath,Jahr,false,dsMain);
  ShowMonat (Sender);
  with Printer do begin
    PrPlot:=TPlot.Create (Canvas);
    PrinterIndex:=PrtNdx;
    if PrtNdx=-1 then PrtNdx:=PrinterIndex;
    with Printer do PrtName:=Printers[PrtNdx];
    with StatusBar do begin
      with Panels[1] do Text:=MinimizeName(DatPath,Canvas,Width);
      Panels[3].Text:=Printer.Printers[PrtNdx]+ OriName[Printer.Orientation];
      end;
    end;
  end;

procedure THauptForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  mmVis:=MinMaxDialog.Visible;
  WriteIni;
  end;

{-----------------------------------------------------------------------}
(* Daten laden *)
function THauptForm.DatenLaden (Pfad : string;
                                Jahr : integer;
                                Neu  : boolean;
                                ADatSet : TDataSet) : boolean;
var
  Name  : string;
begin
  with Daten[ADatSet] do begin
    Name:=Pfad+CreateFileName(Jahr);
    if ReadData (Name,Jahr,DatenBuffer) then begin
      ReadLastPreviousDay(Pfad,Jahr,LastPreviousDay);
      ReadFirstNextDay(Pfad,Jahr,FirstNextDay);
      Loaded:=true;
      DatName:=Name;
      DatJahr:=Jahr;
      ShowAverage(ADatSet);
      Result:=true;
      end
    else begin
      if Neu then DatName:=Name
      else ShowMessage ('Keine Daten fr '+IntToStr(Jahr)+' in '+
                        Pfad+' gefunden!');
      Result:=false;
      end;
    end;
  end;

{-----------------------------------------------------------------------}
(* Daten sichern *)
procedure THauptForm.DatenSichern;
var
  Name     : string;
begin
  with Daten[dsMain] do if Loaded then begin
    Name:=ChangeFileExt(DatName,'.BAK');
    DeleteFile (Name);  (* alter BAK-File *)
    RenameFile (DatName,Name);
    WriteData (DatName,Jahr,DatenBuffer);
    end;
  end;

{-----------------------------------------------------------------------}
(* Gre des Zeichenfeldes festlegen *)
procedure THauptForm.InitPlotField (var Plot     : TPlot;
                                    Rand         : TFRect;
                                    PagePos      : TPagePos);
begin
  with Rand do if PagePos=poOben then begin
    Top:=0.5; Bottom:=0.5*PrSize.Y+0.5;
    end
  else begin
    Top:=0.5*PrSize.Y-0.5; Bottom:=0.5;
    end;
  with Plot do begin
    SetPlotField (PrSize,Rand);  (* Rand fr obere/untere Hlfte *)
    end;
  end;

{-----------------------------------------------------------------------}
// Jahreswerte anzeigen
procedure THauptForm.ShowAverage (ADatSet : TDataSet);
var
  im,id,k : integer;
  tm,rn   : double;
begin
  with Daten[ADatSet] do begin
    tm:=0; rn:=0; k:=0;
    for im:=1 to 12 do begin
      for id:=1 to MonatsTage(im,DatJahr) do with DatenBuffer[im,id] do begin
        if Valid(T,TNoVal) then begin
          tm:=tm+T; inc(k);
          end
        else if Valid(TMin,TNoVal) and Valid(TMax,TNoVal)  then begin
          tm:=tm+(Tmax+TMin)/2; inc(k);
          end;
        if ((im>1) or (id>1)) and Valid(Regen,RNoVal) then
          rn:=rn+Regen;
        end;
      end;
    // Regen vom 1. Jan. des Folgejahres
    rn:=rn+FirstNextDay.Regen;
    if ADatSet=dsMain then begin
      if k>0 then laTemp.Caption:=FloatToStrX(tm/k,5,1)+' C' else laTemp.Caption:='';
      laRain.Caption:=FloatToStrX(rn,5,0)+' mm';
      end
    else if ADatSet=dsComp then with laVJahr do begin
      Caption:=IntToStr(DatJahr)+': ';
      if k>0 then Caption:=Caption+FloatToStrX(tm/k,5,1)+' C / ';
      Caption:=Caption+FloatToStrX(rn,5,0)+' mm';
      end;
    end;
  end;

{-----------------------------------------------------------------------}
(* Wochentag berechnen *)
function Wochentag (Tag,Monat,Jahr : word) : integer;
begin
  Result:=DayOfWeek(EncodeDate(Jahr,Monat,Tag));
  end;

{-----------------------------------------------------------------------}
(* Summe bilden *)
procedure Summe (z       : double;
                 var sum : double;
                 var i   : integer);
begin
  sum:=sum+z; i:=i+1;
  end;

{-----------------------------------------------------------------------}
(* Diagramm malen *)
procedure THauptForm.Diagramm (Plot     : TPlot;
                               var WDaten : TJahresdaten;
                               AJahr,
                               Monat    : word;
                               Clear,
                               WithAxis : boolean);
var
  sum,xt,r,
  x,y,xalt,yalt : double;
  DrawLine,bs   : boolean;
  wdt,
  i,tag,mode,w  : integer;
  s             : string;
  x0,y0,dx0,dy0 : double;
  dr            : TDatRecTyp;

  procedure SetFirstPoint (VNr : integer; PrevVal,FirstVal,NoVal : double);
  begin
    with Plot do if Valid (PrevVal,NoVal) and Valid(FirstVal,NoVal) then begin
      xalt:=UserScale(1,0); yalt:=UserScale(VNr,0.5*(FirstVal-PrevVal)+PrevVal);
      DrawLine:=true;
      end
    else DrawLine:=false;
    end;

  procedure SetLastPoint (VNr : integer; LastVal,NextVal,NoVal : double);
  begin
    with Plot do if Valid (LastVal,NoVal) and Valid(NextVal,NoVal) then begin
      CPlot(UserScale(1,MonatsTage(monat,AJahr)),
            UserScale(VNr,0.5*(NextVal-LastVal)+LastVal),pmDown,mmNoMark);
      end
    end;

  procedure PlotSnowMark (x1,x2,y : double);
  var
    x,d,d30,d60 : double;
    bm          : TBitmap;
  begin
    with Plot do begin
      bm:=TBitmap.Create;
      x:=(x1+x2)/2; d:=USize/2;
      ilSymbols.GetBitmap(0,bm);
      PlotBitmap(x-d,y+d,USize,USize,bm);
      bm.Free;
      end;
    end;

begin
//  WithAxis:=false;
  if Plot=PrPlot then wdt:=2 else wdt:=1;
  with Plot,WDaten do if Loaded then begin
    with PlotField do begin
      x0:=0.5*(Right+Left);
      dx0:=0.5*(Right-Left)-0.5;
      y0:=0.5*(Bottom+Top);
      dy0:=0.5*(Top-Bottom)-0.5;
      end;
    StartPlot(Clear,white);
    SetColor(black);
    CPlot(x0,y0,pmUpOrigin,mmNoMark);
//    DefUserField (ufAbs,-dx0,dx0,-dy0,dy0);      (* Zeichenfeld festlegen *)
    DefUserField (ufAbs,-dx0+1.0,dx0-0.7,0.5,dy0-0.5); // Zeichenfeld festlegen
    if WithAxis then begin
      SetTextHeight (TSize);
      PlotStr(0.0,dy0-0.1,Monatwahl.Items[Monat-1]+' '+Anumb(AJahr,0),
              true,alCenterTop,black);
      SetTextHeight (MSize);
      PlotStr (-dx0,-dy0+0.7,sm,true,alLeftTop,black);
      xt:=GetTextWidth(sm);
      end;
    SetLineWidth(1);
    (* Temperaturen *)
    DefUserScale (1,stHorLin,0.0,31.0);
    with TempRange do DefUserScale (5,stVertLin,min,max);
    if WithAxis then begin
      SetTextHeight (TSize);
      PlotStr (-dx0,dy0-0.3,'T',true,alLeftTop,black);
      SetTextHeight (USize);
      PlotStr (UserScale(1,0.0)-0.3,UserScale(5,TempRange.Min),'C',true,alRightBottom,black);
      SetTextHeight (MSize);
      SetLineStyle (1);
      y:=UserScale(5,0.0); (* 0-Linie *)
      CPlot (UserScale(1,0.0),y,pmUp,mmNoMark); CPlot (UserScale(1,31.0),y,pmDown,mmNoMark);
      y:=UserScale(5,20.0); (* 20-Linie *)
      CPlot (UserScale(1,0.0),y,pmUp,mmNoMark); CPlot (UserScale(1,31.0),y,pmDown,mmNoMark);
      SetLineStyle (0);
      SetAxisProp (0,0.3,0.02,0.2,0.15,0.07);
      XMaxLabel:=DaysInAMonth(AJahr,Monat);
      Axis (1,UserScale(5,TempRange.Min),1,1,[asBottom,asNoZero,asFixed,asCenterText],black);
      Axis (5,UserScale(1,0.0),10.0,10,[asLeft,asFixed],black);
      (* Sonntage markieren*)
      for tag:=1 to MonatsTage(monat,AJahr) do begin
        w:=Wochentag (tag,monat,AJahr);
        if ((w=1) or (w=2)) and (tag>1) then begin
          SetLineStyle (1);
          x:=UserScale(1,tag-1);
          CPlot (x,UserScale(5,Temprange.Min),pmUp,mmNoMark);
          CPlot (x,UserScale(5,TempRange.Max),pmDown,mmNoMark);
          SetLineStyle (0);
          end;
        end;
      PenColor:=LineColors[ltTMin];  (* min. Temp *)
      end
    else begin
      SetTextHeight (MSize);
      SetLineStyle(1); SetColor (lightblue);
      end;
    i:=0; sum:=0.0;
    SetLineWidth(wdt);    // T(min)
    if monat=1 then dr:=LastPreviousDay
    else dr:=DatenBuffer[monat-1,MonatsTage(monat-1,AJahr)];
    SetFirstPoint(5,dr.TMin,DatenBuffer[monat,1].TMin,TNoVal);
    for tag:=1 to MonatsTage(monat,AJahr) do with DatenBuffer[monat,tag] do begin
      if Valid (TMin,TNoVal) then begin
        x:=UserScale(1,tag-0.5); y:=UserScale(5,tmin); summe(tmin,sum,i);
        if DrawLine then begin
          CPlot (xalt,yalt,pmUp,mmNoMark);
          CPlot(x,y,pmDown,mmNoMark);
          end
        else CPlot(x,y,pmUp,mmNoMark);
        DrawLine:=true; xalt:=x; yalt:=y;
        end
      else DrawLine:=false;
      end;
    if monat=12 then dr:=FirstNextDay
    else dr:=DatenBuffer[monat+1,1];
    SetLastPoint(5,DatenBuffer[monat,MonatsTage(monat,AJahr)].TMin,dr.TMin,TNoVal);
    if WithAxis then begin
      if (i<>0) then begin
        s:='T(min) = '+Anumb(sum/i,1)+' C,';
        PlotStr(-dx0+xt,-dy0+0.7,s,true,alLeftTop,blue);
        xt:=xt+GetTextWidth(s);
        end;
      PenColor:=LineColors[ltTemp];
      end
    else SetColor (lightgreen);
    i:=0; sum:=0.0;
    SetLineWidth(wdt);   // T
    if monat=1 then dr:=LastPreviousDay
    else dr:=DatenBuffer[monat-1,MonatsTage(monat-1,AJahr)];
    SetFirstPoint(5,dr.T,DatenBuffer[monat,1].T,TNoVal);
    for tag:=1 to MonatsTage(monat,AJahr) do with DatenBuffer[monat,tag] do begin
      if Valid (T,TNoVal) then begin
        x:=UserScale(1,tag-0.5); y:=UserScale(5,t);
        summe (t,sum,i);
        if DrawLine then begin
          CPlot (xalt,yalt,pmUp,mmNoMark);
          CPlot(x,y,pmDown,mmNoMark);
          end
        else CPlot(x,y,pmUp,mmNoMark);
        DrawLine:=true; xalt:=x; yalt:=y;
        end
      else DrawLine:=false;
      end;
    if monat=12 then dr:=FirstNextDay
    else dr:=DatenBuffer[monat+1,1];
    SetLastPoint(5,DatenBuffer[monat,MonatsTage(monat,AJahr)].T,dr.T,TNoVal);
    if (i<>0) then begin
      s:='  T(Mittel) = '+Anumb(sum/i,1)+' C';
      if WithAxis then begin
        s:=s+',';
        PlotStr (-dx0+xt,-dy0+0.7,s,true,alLeftTop,green);
        xt:=xt+GetTextWidth(s);
        PenColor:=LineColors[ltTMax];
        end
      else begin
        PlotStr (dx0,-dy0+0.7,s,true,alRightTop,green);
        PlotStr (dx0-GetTextWidth(s),-dy0+0.7,IntToStr(WDaten.DatJahr)+':',false,alRightTop,black);
        SetColor (lightred);
        end;
      end;
    i:=0; sum:=0.0;
    SetLineWidth(wdt);    // T(max)
    if monat=1 then dr:=LastPreviousDay
    else dr:=DatenBuffer[monat-1,MonatsTage(monat-1,AJahr)];
    SetFirstPoint(5,dr.TMax,DatenBuffer[monat,1].TMax,TNoVal);
    for tag:=1 to MonatsTage(monat,AJahr) do
        with DatenBuffer[monat,tag] do begin
      if Valid (TMax,TNoVal) then begin
        x:=UserScale(1,tag-0.5); y:=UserScale(5,tmax);
        summe (tmax,sum,i);
        CPlot (xalt,yalt,pmUp,mmNoMark);
        if DrawLine then begin
          CPlot (xalt,yalt,pmUp,mmNoMark);
          CPlot(x,y,pmDown,mmNoMark);
          end
        else CPlot(x,y,pmUp,mmNoMark);
        DrawLine:=true; xalt:=x; yalt:=y;
        end
      else DrawLine:=false;
      end;
    if monat=12 then dr:=FirstNextDay
    else dr:=DatenBuffer[monat+1,1];
    SetLastPoint(5,DatenBuffer[monat,MonatsTage(monat,AJahr)].TMax,dr.TMax,TNoVal);
    if WithAxis and (i<>0) then begin
      s:='  T(max) = '+Anumb(sum/i,1)+' C,';
      PlotStr (-dx0+xt,-dy0+0.7,s,true,alLeftTop,red);
      xt:=xt+GetTextWidth(s);
      end;
    DefUserField (ufAbs,-dx0+1.0,dx0-0.7,-dy0+1.5,0.0);
    SetLineWidth(1);
    (* Luftdruck *)
    DefUserScale (1,stHorLin,0.0,31.0);
    with DruckRange do DefUserScale (5,stVertLin,min,max);
    y:=UserScale(5,1012.0);
    if WithAxis then begin
      SetLineStyle (1); SetColor (black);
      CPlot(UserScale(1,0.0),y,pmUp,mmNoMark);
      CPlot(UserScale(1,31.0),y,pmDown,mmNoMark);
      SetLineStyle (0);
      Axis (1,UserScale(5,DruckRange.Min),1,1,[asBottom,asNoZero,asFixed,asCenterText],black);
      AxisProperties.Digits:=4;
      Axis (5,UserScale(1,0.0),50.0,10,[asLeft,asFixed],black);
      SetTextHeight (TSize);
      PlotStr (-dx0,0.0,'P',true,alLeftBottom,black);
      SetTextHeight (USize);
      PlotStr (UserScale(1,0.0)-0.3,UserScale(5,DruckRange.Min),'hPa',true,alRightBottom,black);
      (* Sonntage markieren*)
      for tag:=1 to MonatsTage(monat,AJahr) do begin
        w:=Wochentag (tag,monat,AJahr);
        if ((w=1) or (w=2)) and (tag>1)  then begin
          SetLineStyle (1);
          x:=UserScale(1,tag-1);
          CPlot (x,UserScale(5,DruckRange.Min),pmUp,mmNoMark);
          CPlot (x,UserScale(5,DruckRange.Max),pmDown,mmNoMark);
          SetLineStyle (0);
          end;
        end;
      PenColor:=LineColors[ltPrs];
      end
    else begin
      SetLineStyle(1); SetColor (lightmagenta);
      end;
    i:=0; sum:=0.0;
    SetLineWidth(wdt);          // Luftdruck
    if monat=1 then dr:=LastPreviousDay
    else dr:=DatenBuffer[monat-1,MonatsTage(monat-1,AJahr)];
    SetFirstPoint(5,dr.Druck,DatenBuffer[monat,1].Druck,DNoVal);
    for tag:=1 to MonatsTage(monat,AJahr) do
        with DatenBuffer[monat,tag] do begin
      if Valid (Druck,DNoVal) then begin
        x:=UserScale(1,tag-0.5); y:=UserScale(5,druck);
        summe (druck,sum,i);
        if DrawLine then begin
          CPlot(xalt,yalt,pmUp,mmNoMark);
          CPlot(x,y,pmDown,mmNoMark);
          end
        else CPlot(x,y,pmUp,mmNoMark);
        DrawLine:=true; xalt:=x; yalt:=y;
       end
       else DrawLine:=false;
     end;
    if monat=12 then dr:=FirstNextDay
    else dr:=DatenBuffer[monat+1,1];
    SetLastPoint(5,DatenBuffer[monat,MonatsTage(monat,AJahr)].Druck,dr.Druck,DNoVal);
    if WithAxis then begin
      SetTextHeight (MSize);
      if i<>0 then PlotStr (-dx0+xt,-dy0+0.7,'  P = '+ANumb(sum/i,0)+' hPa',
               true,alLeftTop,magenta);
      end;
    SetLineWidth(1);
    // Regen
    with RegenRange do DefUserScale(5,stVertLin,min,max);
    if WithAxis then begin
      SetColor (black);
      AxisProperties.Digits:=0;
      Axis (5,UserScale(1,31.0),20.0,10,[asRight,asFixed],black);
      SetTextHeight (TSize);
      PlotStr (dx0,0.0,'N',true,alRightBottom,black);
      SetTextHeight (USize);
      PlotStr (UserScale(1,31.0)+0.3,UserScale(5,RegenRange.Min)+0.2,'mm',true,alLeftBottom,black);
      SetTextHeight (MSize);
      SetLineStyle (0);
      PenColor:=LineColors[ltRain];
      end
    else begin
      SetLineStyle(1); SetColor (lightblue);
      end;
    yalt:=UserScale(5,0.0); mode:=pmUp;
    i:=0; sum:=0.0;
    SetLineWidth(wdt);      // Regen
    for tag:=2 to MonatsTage(monat,AJahr) do
        with DatenBuffer[monat,tag] do begin
      if not Valid (Regen,RNoval) then regen:=0.0;
      xalt:=UserScale(1,tag-2.0); summe (regen,sum,i);
      x:=UserScale(1,tag-1.0); y:=UserScale(5,regen);
      if tag>2 then begin
        CPlot (xalt,yalt,mode,mmNoMark); mode:=pmDown;
        end;
      CPlot(xalt,y,mode,mmNoMark);
      if Schnee then PlotSnowMark(xalt,x,y);
      if regen>=0.1 then begin
        CPlot (x,y,pmDown,mmNoMark); mode:=pmDown;
        end
      else mode:=pmUp;
      yalt:=y;
      end;
    tag:=MonatsTage(monat,AJahr);
    if monat<12 then with DatenBuffer[monat+1,1] do begin
      r:=Regen; bs:=Schnee;
      end
    else with FirstNextDay do begin
      r:=Regen; bs:=Schnee;
      end;
    if not Valid (r,RNoval) then r:=0.0;
    summe (r,sum,i);
    xalt:=UserScale(1,tag-1); y:=UserScale(5,r);
    CPlot(xalt,yalt,mode,mmNoMark);
    CPlot(xalt,y,pmDown,mmNoMark);
    x:=UserScale(1,tag);
    if bs then PlotSnowMark(xalt,x,y);
    if r>0.1 then begin
      CPlot (x,y,pmDown,mmNoMark);
      if tag<31 then CPlot (x,UserScale(5,0.0),pmDown,mmNoMark);
      end;
    if WithAxis then begin   // Monatssumme
      PlotStr(-dx0,-dy0+0.2,sn,true,alLeftTop,black);
      PlotStr(-dx0+GettextWidth(sn),-dy0+0.2,Anumb(sum,1)+' mm',true,alLeftTop,cyan);
      end
    else begin
      PlotStr(dx0,-dy0+0.2,'N = '+ANumb(sum,1)+' mm',false,alRightTop,cyan);
      end;
    SetLineWidth(1);
    end;
  end;

procedure THauptForm.EndeClick(Sender: TObject);
begin
  close;
  end;

procedure THauptForm.GrafikFeldMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  d   : integer;
  lt  : TLineType;
  val : double;
  sh  : string;
  hp  : TPoint;

  function GetDay (ix,iy : integer) : integer;
  var
    MPos : TPoint;
    x,y  : double;
  begin
    MPos.X:=iX; MPos.Y:=iY;
    with GrPlot do begin
      ViewToWindow (MPos);
      if InPlotField(MPos.x,MPos.y) then begin
        InvCScale(MPos.x,MPos.y,x,y);
        x:=InvUserScale(1,x); y:=frac(x);
        if (y<0.1) or (y>0.9) then Result:=0 else Result:=trunc(x)+1;
        end
      else Result:=0;
      end;
    end;

   function GetLineType (ix,iy : integer) : TLineType;
   var
     i : integer;
     col : TColor;
   begin
//     laRain.Caption:=ColorToString(GrPlot.PlotCanvas.Pixels[x,y]);
     for i:=iy-2 to iy+2 do begin
       col:=GrPlot.PlotCanvas.Pixels[ix,i] and $FFFFFF;
       if col<>clWhite then Break;
       end;
     if col<>clWhite then begin
       for Result:=ltTMax to ltRain do if LineColors[Result]=col then Exit;
       end;
     Result:=ltNone;
     end;

begin
  lt:=GetLineType(x,y); sh:='';
  if lt<>ltNone then begin
  //  laRain.Caption:=IntToStr(x)+'/'+IntToStr(y)+' '+ColorToString(GrPlot.PlotCanvas.Pixels[x,y]);
    GrPlot.SetScreenWindow (shNoMargin,GrSize);
    d:=GetDay(x,y);
    if d>0 then with Daten[dsMain] do begin
      case lt of
      ltTMin : sh:=FloatToStrX(DatenBuffer[monat,d].TMin,5,1);
      ltTemp : sh:=FloatToStrX(DatenBuffer[monat,d].T,5,1);
      ltTMax : sh:=FloatToStrX(DatenBuffer[monat,d].TMax,5,1);
      ltPrs : sh:=FloatToStrX(DatenBuffer[monat,d].Druck,5,0);
      ltRain : begin
          if d=MonatsTage(Monat,Jahr) then begin
            if monat<12 then val:=DatenBuffer[monat+1,1].regen
            else val:=FirstNextDay.Regen;
            end
          else val:=DatenBuffer[monat,d+1].regen;
          sh:=FloatToStrX(val,5,1);
          end;
        end;
      end;
    end;
  with bhPlotSheet do if length(sh)>0 then begin
    Description:=sh;
    hp:=GrafikFeld.ClientToScreen(point(x,y));
    with hp do if Y <= Screen.Height div 2 then Y:=Y-40;
    if not ShowingHint then ShowHint(hp);
    end
  else HideHint;
  end;

procedure THauptForm.NeuGraf(Sender: TObject);
begin
  GrPlot.SetScreenWindow (shNoMargin,GrSize);
  if cbCompare.Checked then begin
    Diagramm(GrPlot,Daten[dsComp],Jahr,Monat,true,false);
    Diagramm(GrPlot,Daten[dsMain],Jahr,Monat,false);
    end
  else Diagramm(GrPlot,Daten[dsMain],Jahr,Monat);
  end;

procedure THauptForm.DrEinstell(Sender: TObject);
begin
  PrinterSetupDialog.Execute;
  if (PrtNdx<>Printer.PrinterIndex) then begin
    PrtNdx:=Printer.PrinterIndex;
    Printer.Orientation:=poPortrait;
    GrPlot.GetDefMargins(Printer.Orientation,GrRand);
    with PrSize do GrPlot.GetPaperSize (Printer,X,Y);
    InitPlotField (GrPlot,GrRand,poOben);
    GrafikFeld.Invalidate;
    with Printer do PrtName:=Printers[PrinterIndex];
    StatusBar.Panels[3].Text:=Printer.Printers[PrtNdx]+ OriName[Printer.Orientation];
    end;
  end;

procedure THauptForm.Drucken(Sender: TObject);
var
  PrRand : TFRect;
begin
if ConfirmDialog(CursorPos,ProgName,'Soll der aktuelle Monat ausgedruckt werden ?') then with Printer do begin
  PrRand:=GrRand;
  Title:='Wetter '+IntToStr(Jahr)+'-'+ZStrint(Monat,2);
  BeginDoc;
  InitPlotField (PrPlot,PrRand,poOben);
  PrPlot.SetPrintWindow (shMargin);
  Diagramm (PrPlot,Daten[dsMain],Jahr,Monat);
  EndDoc;
  end;
end;

procedure THauptForm.JahrDrucken(Sender: TObject);
begin
  if JahrWahlDialog.Execute(PJahr) then begin
//  ConfirmDialog(CursorPos,ProgName,Format('Alle Monate des Jahres %u drucken?',[Jahr])) then
     if DatenLaden (DatPath,PJahr,false,dsPrint) then begin
       MonateDrucken (Daten[dsPrint],PJahr,1,12);
       end;
     end;
  end;

procedure THauptForm.TeilDrucken(Sender: TObject);
var
  dt : TDateTime;
  m1,m2 : integer;
begin
  m1:=1; dt:=ToDay;
  if YearOf(dt)=Jahr then m2:=MonthOf(dt) else m2:=12;
  if MonateWahl.GetMonate(Jahr,m1,m2) then MonateDrucken (Daten[dsMain],Jahr,m1,m2);
  end;

procedure THauptForm.MonateDrucken (var ADaten : TJahresDaten; AJahr,Monat1,Monat2 : integer);
var
  m      : integer;
  Pos    : TPagePos;
  PrRand : TFRect;
begin
with Printer do begin
  HauptForm.Enabled:=false;
  with WetHinweis do begin
    Caption:='Drucken';
    Hinweis1.Caption:=Printers[PrtNdx]+#13'Ausdruck fr das Jahr '+IntToStr(Jahr);
    Show;
    end;
  PrRand:=GrRand;
  Title:='Wetter '+IntToStr(AJahr);
  BeginDoc;
  for m:=Monat1 to Monat2 do if not WetHinweis.Abbruch then begin
    with WetHinweis do begin
      Hinweis2.Caption:='Monat: '+MonatWahl.Items[m-1];
      Invalidate;
      end;
    Application.ProcessMessages;
    Pos:=TPagePos((m-1) mod 2);
    InitPlotField (PrPlot,PrRand,Pos);
    PrPlot.SetPrintWindow (shMargin);
    Diagramm (PrPlot,ADaten,AJahr,m,Pos=poOben);
    if (m mod 2 = 0) and (m<12) then NewPage;
    end;
  if WetHinweis.Abbruch then Abort else EndDoc;
  end;
  HauptForm.Enabled:=true;
  WetHinweis.Close;
(*  GrafikFeld.invalidate; *)
  end;

procedure THauptForm.NeuerMonat(Sender: TObject);
begin
  Monat:=succ(MonatWahl.ItemIndex);
  GrafikFeld.invalidate;
  end;

procedure THauptForm.AktMonatClick(Sender: TObject);
var
  dt : TDateTime;
begin
  dt:=ToDay;
  Monat:=MonthOf(dt); Jahr:=YearOf(dt);
  DatenLaden (DatPath,Jahr,false,dsMain);
  GrafikFeld.invalidate;
  ShowMonat(Sender);
  JahrWahl.Text:=IntToStr(Jahr);
  end;

procedure THauptForm.Mon3Zur(Sender: TObject);
var
  j : integer;
begin
  if Monat>3 then dec(Monat,3)
  else begin
    if Monat>1 then Monat:=1
    else begin
      Monat:=12;
      j:=pred(Jahr);
      if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
      JahrWahl.Text:=IntToStr(Jahr);
      end;
    end;
  GrafikFeld.invalidate;
  ShowMonat(Sender);
  end;

procedure THauptForm.Mon3Vor(Sender: TObject);
var
  j : integer;
begin
  if Monat<10 then inc(Monat,3)
  else begin
    if Monat<12 then Monat:=12
    else begin
      Monat:=1;
      j:=succ(Jahr);
      if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
      JahrWahl.Text:=IntToStr(Jahr);
      end;
    end;
  GrafikFeld.invalidate;
  ShowMonat(Sender);
  end;

procedure THauptForm.MonIncr(Sender: TObject);
var
  j : integer;
begin
  if Monat<12 then inc(Monat)
  else begin
    Monat:=1;
    j:=succ(Jahr);
    if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
    JahrWahl.Text:=IntToStr(Jahr);
    end;
  GrafikFeld.invalidate;
  ShowMonat(Sender);
  end;

procedure THauptForm.MonDecr(Sender: TObject);
var
  j : integer;
begin
  if Monat>1 then dec(Monat)
  else begin
    Monat:=12;
    j:=pred(Jahr);
    if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
    JahrWahl.Text:=IntToStr(Jahr);
    end;
  GrafikFeld.invalidate;
  ShowMonat(Sender);
  end;

procedure THauptForm.JanuarClick(Sender: TObject);
var
  j : integer;
begin
  if Monat>1 then Monat:=1
  else begin
    Monat:=12;
    j:=pred(Jahr);
    if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
    JahrWahl.Text:=IntToStr(Jahr);
    end;
  GrafikFeld.invalidate;
  ShowMonat(Sender);
  end;

procedure THauptForm.DezemberClick(Sender: TObject);
var
  j : integer;
begin
  if Monat<12 then Monat:=12
  else begin
    Monat:=1;
    j:=succ(Jahr);
    if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
    JahrWahl.Text:=IntToStr(Jahr);
    end;
  GrafikFeld.Invalidate;
  ShowMonat(Sender);
end;

procedure THauptForm.ShowMonat(Sender: TObject);
begin
  with MonatWahl do begin
    ItemIndex:=pred(Monat);
    Text:=Items[ItemIndex];
    end;
  end;

procedure THauptForm.AnderJahrClick(Sender: TObject);
var
  j,ic : integer;
begin
  val (JahrWahl.Text,j,ic);
  if ic=0 then begin
    if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
    end;
  GrafikFeld.invalidate;
  end;

procedure THauptForm.JahrEingabe(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) or (Key=VK_LBUTTON) then begin
    AnderJahrClick (Sender); MonatWahl.SetFocus;
    end;
  end;

procedure THauptForm.JahrDecr(Sender: TObject);
var
  j : integer;
begin
  j:=pred(Jahr);
  if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
  JahrWahl.Text:=IntToStr(Jahr); MonatWahl.SetFocus;
  GrafikFeld.invalidate;
  end;

procedure THauptForm.JahrInc(Sender: TObject);
var
  j : integer;
begin
  j:=succ(Jahr);
  if DatenLaden (DatPath,j,false,dsMain) then Jahr:=j;
  JahrWahl.Text:=IntToStr(Jahr); MonatWahl.SetFocus;
  GrafikFeld.invalidate;
  end;

procedure THauptForm.ResetChecked (Menu : TMenuItem);
var
  i : integer;
begin
  with Menu do begin
    for i:=0 to Count-1 do Items[i].Checked:=false;
    end;
  end;

procedure THauptForm.Anpassen1Click(Sender: TObject);
begin
  ResetChecked (AnsichtMenu);
  Anpassen1.Checked:=true;
  GrSize:=StSize;
end;

procedure THauptForm.N111Click(Sender: TObject);
begin
  ResetChecked (AnsichtMenu);
  N111.Checked:=true;
  with GrPlot.WindField do begin
    GrSize.X:=Right-Left; GrSize.y:=Top-Bottom;
    end;
end;

procedure THauptForm.N121Click(Sender: TObject);
begin
  ResetChecked (AnsichtMenu);
  N121.Checked:=true;
  with GrPlot.WindField do begin
    GrSize.x:=(Right-Left) div 2; GrSize.y:=(Top-Bottom) div 2;
    end;
end;

procedure THauptForm.N141Click(Sender: TObject);
begin
  ResetChecked (AnsichtMenu);
  N141.Checked:=true;
  with GrPlot.WindField do begin
    GrSize.x:=(Right-Left) div 4; GrSize.y:=(Top-Bottom) div 4;
    end;
end;

procedure THauptForm.SetboxSizes(Sender: TObject);
begin
  with GrafikBox do begin
    StSize.X:=Width-2;
    StSize.Y:=Height-2;
    end;
  if Anpassen1.Checked then GrSize:=StSize;
  end;

procedure THauptForm.PfadSetzen(Sender: TObject);
begin
  if ShellDirDialog.Execute ('Verzeichnis fr Wetterdaten',true,true,false,DocPath,DatPath) then begin
    DatPath:=SetDirName(DatPath);
    with StatusBar.Panels[1] do Text:=MinimizeName(DatPath,Canvas,Width);
    DatenLaden (DatPath,Jahr,false,dsMain);
    GrafikFeld.Invalidate;
    end;
  end;

procedure THauptForm.InfoMessage(Sender: TObject);
begin
  InfoDialog(CursorPos(Point(25,50)),ProgName,ProgVersName+' - '+ProgVersDate+#13+Versinfo.Copyright+#13+'E-Mail: '+EMailAdr);
  end;

procedure THauptForm.NeuesJahrClick(Sender: TObject);
var
  j : word;
  ok : boolean;
begin
  j:=Neueingabe.GetJahr(succ(Jahr));
  if (j>0) then begin
    if (j=Jahr) or DatenLaden (DatPath,j,true,dsMain) then begin
      Jahr:=j;
      ok:=Application.MessageBox (
       'Fr dieses Jahr gibt es bereits Daten!'#13'Sollen sie gelscht werden?',
       'Warnung',MB_YESNO+MB_ICONQUESTION+MB_DEFBUTTON2)= IDYES;
      end
    else ok:=true;
    if ok then with WetEditor do begin
      LTag:=1; Monat:=1;
      ClearBuffer (j,ArbeitsBuffer);
      if StartEdit(LTag,Monat,j) then begin
        Jahr:=j;
        with Daten[dsMain] do begin
          DatenBuffer:=ArbeitsBuffer;
          ReadLastPreviousDay(DatPath,Jahr,LastPreviousDay);
          ReadFirstNextDay(DatPath,Jahr,FirstNextDay);
          DatenSichern;
          Loaded:=true;
          end;
        end;
      end;
    JahrWahl.Text:=IntToStr(Jahr);
    ShowAverage(dsMain);
    GrafikFeld.invalidate;
    end;
end;

procedure THauptForm.DataEdit(Sender: TObject);
begin
  with Daten[dsMain] do if Loaded then with WetEditor do begin
    ArbeitsBuffer:=DatenBuffer;
    if StartEdit(LTag,Monat,Jahr) then begin
      DatenBuffer:=ArbeitsBuffer;
      DatenSichern;
      end;
    ShowAverage(dsMain);
    GrafikFeld.invalidate;
    ShowMonat(Sender);
    end;
  end;

procedure THauptForm.itmSaveClick(Sender: TObject);
begin
  with Daten[dsMain] do if Loaded then with WetEditor do DatenSichern;
  end;

procedure THauptForm.ListeAnzeigen(Sender: TObject);
var
  j : word;
  ed : boolean;
begin
  Liste.Caption:='Wetterdaten fr '+MonatWahl.Items[Monat-1]+'/'+IntToStr(Jahr);
  Liste.ShowList (Daten[dsMain].Datenbuffer,Monat,Jahr,LTag,j,ed);
  if ed then DataEdit(Sender);
  end;

procedure THauptForm.MonatAnzeigen (m,j : integer);
begin
  if DatenLaden (DatPath,j,false,dsMain) then begin
    Jahr:=j;
    if m=0 then m:=1;
    Monat:=m;
    JahrWahl.Text:=IntToStr(Jahr);
    ShowMonat(self);
    GrafikFeld.Invalidate;
    end;
  end;

procedure THauptForm.MinMax (AShow : boolean);
begin
  with MinMaxDialog do begin
    if AShow then ShowData(DatPath,Monat,Jahr)
    else if Visible then Close;
    end;
  end;

procedure THauptForm.itmMinMaxClick(Sender: TObject);
begin
  with MinMaxView do begin
    Down:=not Down;
    itmMinMax.Checked:=Down;
    MinMax(Down);
    end;
  end;

procedure THauptForm.MinMaxViewClick(Sender: TObject);
begin
  with MinMaxView do begin
    itmMinMax.Checked:=Down;
    MinMax(Down);
    end;
  end;

procedure THauptForm.MinMaxClosed;
begin
  MinMaxView.Down:=false;
  itmMinMax.Checked:=false;
  end;

procedure THauptForm.EnableCompare (AEnabled : boolean);
begin
  VJahrZur.Enabled:=AEnabled;
  VJahrVor.Enabled:=AEnabled;
  VJahrWahl.Enabled:=AEnabled;
  laVJahr.Visible:=AEnabled;
  end;

procedure THauptForm.cbCompareClick(Sender: TObject);
var
  j,ic : integer;
begin
  EnableCompare(cbCompare.Checked);
  if cbCompare.Checked then begin
    val (VJahrWahl.Text,j,ic);
    if ic=0 then begin
      if DatenLaden (DatPath,j,false,dsComp) then VJahr:=j;
      end;
//    with VJahrWahl do AddItem(IntToStr(VJahr));
    end;
  GrafikFeld.Invalidate;
  end;

procedure THauptForm.VJahrVorClick(Sender: TObject);
var
  j : integer;
begin
  j:=succ(VJahr);
  if DatenLaden (DatPath,j,false,dsComp) then VJahr:=j;
  VJahrWahl.Text:=IntToStr(VJahr);
  GrafikFeld.Invalidate;
  end;

procedure THauptForm.VJahrWahlKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) or (Key=VK_LBUTTON) then cbCompareClick(Sender);
  end;

procedure THauptForm.VJahrZurClick(Sender: TObject);
var
  j : integer;
begin
  j:=pred(VJahr);
  if DatenLaden (DatPath,j,false,dsComp) then VJahr:=j;
  VJahrWahl.Text:=IntToStr(VJahr);
  GrafikFeld.Invalidate;
  end;


end.
